use domain::services::service::ServiceError;
use domain::models::todo::{CreateTodo, Todo};
use domain::repositories::repository::ResultPaging;
use domain::repositories::todo::TodoRepository;
use domain::services::service::ServiceResult;
use domain::services::todo::TodoService;
use domain::models::Id;

#[derive(Clone)]
pub struct TodoServiceImpl<R>
where
    R: TodoRepository
{
    pub repository: R
}

impl<R> TodoServiceImpl<R>
where
    R: TodoRepository
{
    pub fn new(repository: R) -> Self {
        Self {
            repository
        }
    }
}

impl<R> TodoService for TodoServiceImpl<R>
where
    R: TodoRepository
{
    async fn create(&self, create_todo: CreateTodo) -> ServiceResult<Todo> {
        let mut cloned = create_todo.clone();
        self.repository
            .create(&mut cloned)
            .await
            .map_err(|e| -> ServiceError { e.into() })
    }

    async fn list(&self) -> ServiceResult<ResultPaging<Todo>> {
        self.repository
            .list()
            .await
            .map_err(|e| -> ServiceError { e.into() })
    }

    async fn get(&self, id: Id) -> ServiceResult<Todo> {
        self.repository
            .get(id)
            .await
            .map_err(|e| -> ServiceError { e.into() })
    }

    async fn delete(&self, id: Id) -> ServiceResult<()> {
        self.repository
            .delete(id)
            .await
            .map_err(|e| -> ServiceError { e.into() })
    }

    async fn update(&self, todo: Todo) -> ServiceResult<Todo> {
        let mut cloned = todo.clone();
        self.repository
            .update(&mut cloned)
            .await
            .map_err(|e| -> ServiceError { e.into() })
    }
}
