use domain::repositories::repository::{RepositoryError, RepositoryObject, RepositoryOperation};

#[derive(Debug)]
pub struct PostgresRepositoryError(RepositoryError);

impl PostgresRepositoryError {
    pub fn into_inner(self) -> RepositoryError {
        self.0
    }
}

impl From<(sqlx::Error, RepositoryOperation, RepositoryObject)> for PostgresRepositoryError {
    fn from(error_tuple: (sqlx::Error, RepositoryOperation, RepositoryObject)) -> PostgresRepositoryError {

        let operation = match error_tuple.1 {
            RepositoryOperation::Create => "create",
            RepositoryOperation::GetOne => "get",
            RepositoryOperation::GetMany => "list",
            RepositoryOperation::Update => "update",
            RepositoryOperation::Delete => "delete"
        };

        let object = match error_tuple.2 {
            RepositoryObject::Todo => "Todo"
        };

        let message = format!("Could not {} {}: {}", operation, object, error_tuple.0);

        PostgresRepositoryError(
            RepositoryError {
                operation: error_tuple.1,
                object: error_tuple.2,
                message: message.to_string(),
            }
        )
    }
}