use crate::models::{Id, IdParseError};
use serde::Serialize;
use thiserror::Error;

#[derive(Clone, Serialize)]
pub struct Todo {
    pub id: Id,
    pub title: String,
    pub description: String,
    pub completed: bool,
}

#[derive(Clone)]
pub struct CreateTodo {
    pub title: String,
    pub description: String,
}

#[derive(Error, Debug)]
pub enum InvalidCreateTodoError {
    #[error("title parse error")]
    Title,
    #[error("description parse error")]
    Description,
}

#[derive(Error, Debug)]
pub enum InvalidTodoError {
    #[error("id parse error")]
    Id
}

impl Todo {

    pub fn new(id: Id, title: String, description: String, completed: bool) -> Self {
        Self { id, title, description, completed }
    }

    pub fn id(&self) -> Id {
        self.id.clone()
    }

    pub fn title(&self) -> String {
        self.title.clone()
    }

    pub fn description(&self) -> String {
        self.description.clone()
    }

    pub fn completed(&self) -> bool {
        self.completed
    }

}

impl CreateTodo {

    pub fn new(title: String, description: String) -> Self {
        Self { title, description }
    }

    pub fn title(&self) -> String {
        self.title.clone()
    }

    pub fn description(&self) -> String {
        self.description.clone()
    }

}

impl From<IdParseError> for InvalidTodoError {
    fn from(_: IdParseError) -> Self {
        InvalidTodoError::Id
    }
}
