use std::future::Future;

use crate::repositories::repository::{ResultPaging, RepositoryResult};
use crate::models::todo::{Todo, CreateTodo};
use crate::models::Id;

pub trait TodoRepository: Clone + Send + Sync + 'static {
    fn create(self: &Self, new_todo: &CreateTodo) -> impl Future<Output = RepositoryResult<Todo>> + Send;
    fn list(self: &Self) -> impl Future<Output = RepositoryResult<ResultPaging<Todo>>> + Send;
    fn get(self: &Self, id: Id) -> impl Future<Output = RepositoryResult<Todo>> + Send;
    fn delete(self: &Self, id: Id) -> impl Future<Output = RepositoryResult<()>> + Send;
    fn update(self: &Self, todo: &Todo) -> impl Future<Output = RepositoryResult<Todo>> + Send;
}