use std::future::Future;

use crate::models::Id;
use crate::services::service::ServiceResult;
use crate::models::todo::{CreateTodo, Todo};
use crate::repositories::repository::ResultPaging;

pub trait TodoService: Send + Sync + Clone + 'static {
    fn create(&self, todo: CreateTodo) -> impl Future<Output = ServiceResult<Todo>> + Send;
    fn list(&self) -> impl Future<Output = ServiceResult<ResultPaging<Todo>>> + Send;
    fn get(&self, id: Id) -> impl Future<Output = ServiceResult<Todo>> + Send;
    fn delete(&self, id: Id) -> impl Future<Output = ServiceResult<()>> + Send;
    fn update(&self, todo: Todo) -> impl Future<Output = ServiceResult<Todo>> + Send;
}