use axum::extract::{Path, State};

use application::state::AppState;
use domain::models::Id;
use domain::services::todo::TodoService;
use crate::models::error::HttpError;

/// Create a new [Todo].
///
/// # Responses
///
/// - 201 Created: the [Todo] was successfully created.
/// - 422 Unprocessable entity: An [Todo] with the same name already exists.
pub async fn delete_by_id<S: TodoService>(
    State(state): State<AppState<S>>,
    Path(id): Path<String>,
) -> Result<(), HttpError> {

    state.todo_service
        .delete(Id::from_string(id)?)
        .await
        .map_err(|e| e.into())
        .map(|_| ())

}
