use axum::extract::State;

use application::state::AppState;
use domain::models::todo::Todo;
use domain::services::todo::TodoService;
use crate::models::error::HttpError;
use crate::models::response::HttpGetManyResponse;

/// Get all [Todo]s.
///
/// # Responses
///
/// - 201 Created: the [Todo] was successfully created.
/// - 422 Unprocessable entity: An [Todo] with the same name already exists.
pub async fn get_all<S: TodoService>(
    State(state): State<AppState<S>>
) -> Result<HttpGetManyResponse<Todo>, HttpError> {

    state.todo_service
        .list()
        .await
        .map_err(|e| e.into())
        .map(|result| result.into())
}
