use axum::extract::State;
use axum::Json;

use application::state::AppState;
use domain::models::todo::Todo;
use domain::services::todo::TodoService;
use crate::models::error::HttpError;
use crate::models::response::HttpGetOneResponse;
use crate::models::todo::HttpTodoRequest;

/// Update an existing [Todo].
///
/// # Responses
///
/// - 201 Created: the [Todo] was successfully updated.
/// - 422 Unprocessable entity: An [Todo] with the same name already exists.
pub async fn update_by_id<S: TodoService>(
    State(state): State<AppState<S>>,
    Json(req): Json<HttpTodoRequest>,
) -> Result<HttpGetOneResponse<Todo>, HttpError> {

    state.todo_service
        .update(req.try_into()?)
        .await
        .map_err(|e| e.into())
        .map(|todo: Todo| todo.into())
}
