use serde::Serialize;
use serde_json::json;
use axum::Json;
use axum::http::StatusCode;
use axum::response::{IntoResponse, Response};

use domain::models::IdParseError;
use domain::services::service::ServiceError;

/// The response body data field for an unsuccessful request.
#[derive(Debug, Clone, PartialEq, Eq, Serialize)]
pub struct HttpError {
    pub message: String
}

impl From<ServiceError> for HttpError {
    fn from(from: ServiceError) -> Self {
        Self {
            message: from.message
        }
    }
}

/// Converts a [IdParseError] into a [HttpError]
impl From<IdParseError> for HttpError {
    fn from(_: IdParseError) -> Self {
        Self {
            message: "Not a valid ID".to_string()
        }
    }
}

impl IntoResponse for HttpError {
    fn into_response(self) -> Response {
        let http_status_code = StatusCode::INTERNAL_SERVER_ERROR;
        let body = Json(json!({"message": self.message }));
        (http_status_code, body).into_response()
    }
}