use serde::Serialize;
use serde_json::json;
use axum::Json;
use axum::http::StatusCode;
use axum::response::{IntoResponse, Response};

use domain::repositories::repository::ResultPaging;

/// The response body data field for successful response for GET API calls that expect one item to be returned
#[derive(Debug, Clone, PartialEq, Eq, Serialize)]
pub struct HttpGetOneResponse<T>(T);

impl<T> From<T> for HttpGetOneResponse<T> {
    fn from(from: T) -> Self {
        Self {
            0: from
        }
    }
}

impl<T> IntoResponse for HttpGetOneResponse<T>
where 
    T: Serialize
{
    fn into_response(self) -> Response {
        let http_status_code = StatusCode::FOUND;
        let body = Json(json!({"data": self }));
        (http_status_code, body).into_response()
    }
}

/// Get Many Response
/// 

/// The response body data field for successful response for GET API calls that expect lists of many items to be returned
#[derive(Debug, Clone, PartialEq, Eq, Serialize)]
pub struct HttpGetManyResponse<T>(ResultPaging<T>);

impl<T> From<ResultPaging<T>> for HttpGetManyResponse<T> {
    fn from(from: ResultPaging<T>) -> Self {
        Self {
            0: from
        }
    }
}

impl<T> IntoResponse for HttpGetManyResponse<T>
where 
    T: Serialize
{
    fn into_response(self) -> Response {
        let http_status_code = StatusCode::FOUND;
        let body = Json(json!({"data": self }));
        (http_status_code, body).into_response()
    }
}