use serde::Deserialize;

use crate::models::error::HttpError;
use domain::models::Id;
use domain::models::todo::{CreateTodo, Todo};

/// Create Request
/// 

/// The body of an [Todo] creation request.
#[derive(Debug, Clone, PartialEq, Eq, Deserialize)]
pub struct HttpCreateTodoRequest {
    pub title: String,
    pub description: String
}

/// Converts a [CreateTodoHttpRequest] request into a domain [CreateTodo] request
impl From<HttpCreateTodoRequest> for CreateTodo {
    fn from(req: HttpCreateTodoRequest) -> Self {
        Self {
            title: req.title,
            description: req.description
        }
    }
}

/// Update Request
/// 

/// The body of an [Todo] update request.
#[derive(Debug, Clone, PartialEq, Eq, Deserialize)]
pub struct HttpTodoRequest {
    pub id: String,
    pub title: String,
    pub description: String,
    pub completed: bool
}

/// Converts a [UpdateTodoHttpRequest] request into a domain [Todo] request
impl TryFrom<HttpTodoRequest> for Todo {
    type Error = HttpError;
    fn try_from(req: HttpTodoRequest) -> Result<Self, Self::Error> {
        Id::from_string(req.id)
            .map_err(|e| e.into())
            .map(|id| Todo {
                id,
                title: req.title,
                description: req.description,
                completed: req.completed
            })
    }
}
