use crate::handlers::todo::create::create;
use crate::handlers::todo::get_by_id::get_by_id;
use crate::handlers::todo::get_all::get_all;
use crate::handlers::todo::delete_by_id::delete_by_id;
use crate::handlers::todo::update_by_id::update_by_id;
use application::state::AppState;
use axum::routing::get;
use axum::Router;
use domain::services::todo::TodoService;

pub fn api_routes<S: TodoService>() -> Router<AppState<S>> {
    Router::new()
        .route(
            "/todos",
            get(get_all::<S>)
                .post(create::<S>)
                .patch(update_by_id::<S>)
        )
        .route(
            "/todos/:id",
            get(get_by_id::<S>)
                .delete(delete_by_id::<S>),
        )
}
