use std::sync::Arc;

use sqlx::postgres::PgPoolOptions;

use crate::config::Config;
use db::repositories::todo::TodoPostgresRepository;
use application::services::todo::TodoServiceImpl;
use http::server::{HttpServer, HttpServerConfig};

pub async fn run() -> anyhow::Result<()> {

    let config = Config::from_env()?;

    let db_url = std::env::var("DATABASE_URL").expect("DATABASE_URL must be set");

    let pool = PgPoolOptions::new().connect(&db_url).await?;

    let db = Arc::new(pool);

    let todo_repository = TodoPostgresRepository::new(db);
    let todo_service = TodoServiceImpl::new(todo_repository);

    let server_config = HttpServerConfig {
        port: &config.server_port,
    };
    let http_server = HttpServer::new(todo_service, server_config).await?;
    http_server.run().await

}